/* BdfServer - Copyright (c) 2011-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.balayage;

import fr.exemole.bdfserver.api.balayage.BalayageConstants;
import fr.exemole.bdfserver.api.balayage.BalayageUnit;
import fr.exemole.bdfserver.api.balayage.Output;
import fr.exemole.bdfserver.api.balayage.OutputParam;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.localisation.LangContext;
import net.mapeadores.util.text.AccoladePattern;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class OutputBuilder implements BalayageConstants {

    private final InternalOutput output = new InternalOutput();
    private final List<InternalOutputParam> outputParamList = new ArrayList<InternalOutputParam>();

    public OutputBuilder() {
    }

    public void setName(String name) {
        output.setName(name);
    }

    public void setXsltPath(String xsltPath) {
        output.setXsltPath(xsltPath);
    }

    public void setMode(String mode) {
        output.setMode(mode);
    }

    public void setCleanBefore(boolean cleanBefore) {
        output.setCleanBefore(cleanBefore);
    }

    public void setLangOption(short langOption) {
        output.setLangOption(langOption);
    }

    public void setIncludeOutput(boolean includeOutput) {
        output.setIncludeOutput(includeOutput);
    }

    public void setPattern(AccoladePattern pattern) {
        output.setPattern(pattern);
    }

    public void setOutputPathPattern(AccoladePattern outputPathPattern) {
        output.setOutputPathPattern(outputPathPattern);
    }

    public void setLangContext(LangContext langContext) {
        output.setLangContext(langContext);
    }

    public void setPosttransform(String posttransform) {
        output.setPosttransform(posttransform);
    }

    public void addOutputParam(String name, String value) {
        outputParamList.add(new InternalOutputParam(name, value));
    }

    public Output toOutput(BalayageUnit balayageUnit) {
        output.setBalayageUnit(balayageUnit);
        int size = outputParamList.size();
        if (size > 0) {
            OutputParam[] outputParamArray = new OutputParam[size];
            output.setOutputParamArray(outputParamList.toArray(outputParamArray));
        }
        return output;
    }


    private static class InternalOutput implements Output {

        private String name;
        private short langOption = LANG_OPTION_UNKNOWN;
        private String xsltPath;
        private AccoladePattern pattern;
        private AccoladePattern outputPathPattern;
        private String reversePath = "";
        private OutputParam[] outputParamArray;
        private String mode;
        private boolean cleanBefore = false;
        private boolean includeOutput = false;
        private LangContext langContext;
        private BalayageUnit balayageUnit;
        private String posttransform;

        private InternalOutput() {
        }

        @Override
        public String getName() {
            return name;
        }

        @Override
        public String getXsltPath() {
            return xsltPath;
        }

        @Override
        public AccoladePattern getPattern() {
            return pattern;
        }

        @Override
        public AccoladePattern getOutputPath() {
            return outputPathPattern;
        }

        @Override
        public int getOutputParamCount() {
            if (outputParamArray == null) {
                return 0;
            }
            return outputParamArray.length;
        }

        @Override
        public OutputParam getOutputParam(int i) {
            return outputParamArray[i];
        }

        @Override
        public String getMode() {
            return mode;
        }

        @Override
        public boolean isCleanBefore() {
            return cleanBefore;
        }

        @Override
        public short getLangOption() {
            return langOption;
        }

        @Override
        public String getReversePath() {
            return reversePath;
        }

        @Override
        public boolean isIncludeOutput() {
            return includeOutput;
        }

        @Override
        public LangContext getLangContext() {
            return langContext;
        }

        @Override
        public BalayageUnit getBalayageUnit() {
            return balayageUnit;
        }

        @Override
        public String getPosttransform() {
            return posttransform;
        }

        private void setName(String name) {
            this.name = name;
        }

        private void setXsltPath(String xsltPath) {
            this.xsltPath = xsltPath;
        }

        private void setMode(String mode) {
            this.mode = mode;
        }

        private void setCleanBefore(boolean cleanBefore) {
            this.cleanBefore = cleanBefore;
        }

        private void setLangOption(short langOption) {
            this.langOption = langOption;
        }

        private void setIncludeOutput(boolean includeOutput) {
            this.includeOutput = includeOutput;
        }

        private void setPattern(AccoladePattern pattern) {
            this.pattern = pattern;
        }

        private void setOutputPathPattern(AccoladePattern outputPathPattern) {
            this.outputPathPattern = outputPathPattern;
            initReversePath();
        }

        private void setOutputParamArray(OutputParam[] outputParamArray) {
            this.outputParamArray = outputParamArray;
        }

        private void setLangContext(LangContext langContext) {
            this.langContext = langContext;
        }

        private void setPosttransform(String posttransform) {
            this.posttransform = posttransform;
        }

        private void initReversePath() {
            if (outputPathPattern != null) {
                reversePath = StringUtils.getReversePath(outputPathPattern.toString());
            } else {
                reversePath = "";
            }
        }

        private void setBalayageUnit(BalayageUnit balayageUnit) {
            this.balayageUnit = balayageUnit;
        }

    }


    private static class InternalOutputParam implements OutputParam {

        private final String name;
        private final String value;

        private InternalOutputParam(String name, String value) {
            this.name = name;
            this.value = value;
        }

        @Override
        public String getName() {
            return name;
        }

        @Override
        public String getValue() {
            return value;
        }

    }

}
